@@ -5,6 +5,23 @@ from .aes_crypt import AESCrypt
 import os
 import time
 import json
+import re
+
+
+def get_file_list(repo):
+    def get_file_names(contents):
+        """
+        A generator that recursively retrieves file names from a GitHub repository.
+        """
+        for file_content in contents:
+            if file_content.type == "dir":
+                contents.extend(repo.get_contents(file_content.path))
+            else:
+                file_name = re.sub(r"^.+\/", "", file_content.path)
+                yield file_name
+
+    all_files = list(get_file_names(repo.get_contents("")))
+    return all_files
 
 
 def get_github_repo(token, repo_name):
@@ -24,7 +41,7 @@ def decrypt_data_from_github(filename, my_password, github_token, repo_name):
     aes = AESCrypt(my_password)
 
     # Download encrypted file from Github
-    download_file(repo=repo, filename=filename)
+    download_file(repo=repo, github_file_path=filename)
 
     # Read encrypted data from file
     with open(filename, "rb") as f:
@@ -40,40 +57,34 @@ def decrypt_data_from_github(filename, my_password, github_token, repo_name):
 
 
 @retry(exceptions=Exception, tries=3, delay=2, backoff=2)
-def download_file(repo, filename):
-    """
-    從指定的 Github 倉庫下載檔案並保存在本地
-    """
-    cwd = os.path.abspath(os.getcwd())
-    file_content = repo.get_contents(filename)
+def download_file(repo, github_file_path, download_path=None):
+    filename = os.path.basename(github_file_path)
+    if not download_path:
+        download_path = os.getcwd()
+
+    file_content = repo.get_contents(github_file_path)
     file_content_data = file_content.decoded_content
     mode = "wb" if isinstance(file_content_data, bytes) else "w"
-    with open(os.path.join(cwd, filename), mode) as f:
+    with open(os.path.join(download_path, filename), mode) as f:
         f.write(file_content_data)
-    print(f"{filename} downloaded")
+    print(f"{filename} downloaded to {download_path}")
 
 
 @retry(exceptions=Exception, tries=3, delay=2, backoff=2)
 def update_file(repo, filename):
-    """
-    更新指定的 Github 倉庫中的檔案
-    """
     cwd = os.path.abspath(os.getcwd())
-    all_files = []
-    contents = repo.get_contents("")
-    while contents:
-        file_content = contents.pop(0)
-        if file_content.type == "dir":
-            contents.extend(repo.get_contents(file_content.path))
-        else:
-            file = file_content
-            all_files.append(file.path)
+    try:
+        contents = repo.get_contents(filename)
+        sha = contents.sha
+        print(f"{filename} EXISTS")
+    except Exception as e:
+        sha = None
+        print(f"{filename} DOES NOT EXIST")
 
-    # 定義 content 變數，然後讀取本地檔案的內容
-    content = None
     with open(filename, "rb") as f:
         file_content = f.read()
 
+    content = None
     if isinstance(file_content, str):
         with open(os.path.join(cwd, filename), "r") as f:
             content = f.read()
@@ -81,9 +92,12 @@ def update_file(repo, filename):
         with open(os.path.join(cwd, filename), "rb") as f:
             content = f.read()
 
-    # 上傳檔案到 Github 倉庫
-    repo.update_file(filename, f"Update {filename}", content, file.sha)
-    print(f"{filename} updated")
+    if sha:
+        repo.update_file(filename, "committing files", content, sha)
+        print(f"{filename} UPDATED")
+    else:
+        repo.create_file(filename, "committing files", content)
+        print(f"{filename} CREATED")
 
 
 @retry(exceptions=Exception, tries=3, delay=2, backoff=2)
